#uses "fwUnitTestComponentAsserts.ctl"

#uses "test/fwRPC/utils.ctl"

#uses "fwRPC/fwRPC_workerModel.ctl"


// Mocks / utils
//------------------------------------------------------------------------------

const string w_id = "1234";


// setup
//------------------------------------------------------------------------------

void test_fwRPC_workerModel_setupSuite() {
  _BEGIN_SUITE();
}

void test_fwRPC_workerModel_teardownSuite() {
  _END_SUITE();
}

void test_fwRPC_workerModel_setup() {
  _BEGIN_TEST();
}

void test_fwRPC_workerModel_teardown() {
}


// tests
//------------------------------------------------------------------------------

void test_fwRPC_workerModel_createsValidObject() {
  dyn_mixed obj = fwRPC_workerModel_new(w_id);

  assertEqual(fwRPC_WORKER_MODEL_MAXIDX, dynlen(obj));
  assertEqual(w_id, obj[fwRPC_WORKER_MODEL_ID]);
  assertEqual(fwRPC_WORKER_STATUS_NONE, obj[fwRPC_WORKER_MODEL_STATUS]);
  assertEqual(0, obj[fwRPC_WORKER_MODEL_JOBS]);
  assertEqual(false, obj[fwRPC_WORKER_MODEL_LOCKED]);
  assertEqual(false, obj[fwRPC_WORKER_MODEL_PEND_RESTART]);
}


void test_fwRPC_workerModel_checksValidity() {
  dyn_mixed obj = fwRPC_workerModel_new(w_id);

  assertTrue(fwRPC_workerModel_isValid(obj), "worker should be valid");

  obj = makeDynMixed(1);
  assertFalse(fwRPC_workerModel_isValid(obj), "worker should not be valid");
}


void test_fwRPC_workerModel_checks_if_worker_is_suitable() {
  dyn_mixed worker = fwRPC_workerModel_new(w_id, fwRPC_WORKER_STATUS_READY);
  
  assertTrue(fwRPC_workerModel_canProcess(worker, false), "1");
  assertTrue(fwRPC_workerModel_canProcess(worker, true), "2");

  worker[fwRPC_WORKER_MODEL_STATUS] = fwRPC_WORKER_STATUS_ERROR;
  assertFalse(fwRPC_workerModel_canProcess(worker, false), "3");
  assertFalse(fwRPC_workerModel_canProcess(worker, true), "4");

  worker[fwRPC_WORKER_MODEL_STATUS] = fwRPC_WORKER_STATUS_DEAD;
  assertFalse(fwRPC_workerModel_canProcess(worker, false), "5");
  assertFalse(fwRPC_workerModel_canProcess(worker, true), "6");

  worker[fwRPC_WORKER_MODEL_STATUS] = fwRPC_WORKER_STATUS_WORKING;
  assertTrue(fwRPC_workerModel_canProcess(worker, false), "7");
  assertFalse(fwRPC_workerModel_canProcess(worker, true), "8");
  worker[fwRPC_WORKER_MODEL_STATUS] = fwRPC_WORKER_STATUS_READY;

  worker[fwRPC_WORKER_MODEL_JOBS] = fwRPC_WORKER_MAX_JOBS;
  assertFalse(fwRPC_workerModel_canProcess(worker, false), "9");
  assertFalse(fwRPC_workerModel_canProcess(worker, true), "10");
  worker[fwRPC_WORKER_MODEL_JOBS] = 0;

  worker[fwRPC_WORKER_MODEL_LOCKED] = true;
  assertFalse(fwRPC_workerModel_canProcess(worker, false), "13");
  assertFalse(fwRPC_workerModel_canProcess(worker, true), "14");
  worker[fwRPC_WORKER_MODEL_LOCKED] = false;
}
